@@ -1,9 +1,9 @@
 ##### Credits
 
 # ===== Anime Game Remap (AG Remap) =====
-# Authors: NK#1321, Albert Gold#2696
+# Authors: Albert Gold#2696, NK#1321
 #
-# if you used it to remap your mods pls give credit for "Nhok0169" and "Albert Gold#2696"
+# if you used it to remap your mods pls give credit for "Albert Gold#2696" and "Nhok0169"
 # Special Thanks:
 #   nguen#2011 (for support)
 #   SilentNightSound#7430 (for internal knowdege so wrote the blendCorrection code)
@@ -42,6 +42,11 @@ class FileTypes(Enum):
     Blend.buf files
     """
 
+    Position = f"Position{FileExt.Buf.value}"
+    """
+    Position.buf files
+    """
+
     Texture = f"*{FileExt.DDS.value}"
     """
     Texture .dds files
@@ -52,6 +57,11 @@ class FileTypes(Enum):
     RemapBlend.buf files    
     """
 
+    RemapPosition = f"Remap{Position}"
+    """
+    RemapPostion.buf files
+    """
+
     Log = f"RemapFixLog{FileExt.Txt.value}"
     """
     Log file
@@ -61,4 +71,9 @@ class FileTypes(Enum):
     """
     RemapTex.dds files
     """
+
+    RemapDownload = f"RemapDL download"
+    """
+    RemapDL download files
+    """
 ##### EndScript
\ No newline at end of file
