@@ -1,9 +1,9 @@
 ##### Credits
 
 # ===== Anime Game Remap (AG Remap) =====
-# Authors: NK#1321, Albert Gold#2696
+# Authors: Albert Gold#2696, NK#1321
 #
-# if you used it to remap your mods pls give credit for "Nhok0169" and "Albert Gold#2696"
+# if you used it to remap your mods pls give credit for "Albert Gold#2696" and "Nhok0169"
 # Special Thanks:
 #   nguen#2011 (for support)
 #   SilentNightSound#7430 (for internal knowdege so wrote the blendCorrection code)
@@ -36,4 +36,9 @@ class ImgFormats(Enum):
     """
     HSV (hue, saturation, value) image
     """
+
+    Bit = "1"
+    """
+    Image with a single bit channel that has values of either 0 or 1
+    """
 ##### EndScript
\ No newline at end of file
