@@ -1,9 +1,9 @@
 ##### Credits
 
 # ===== Anime Game Remap (AG Remap) =====
-# Authors: NK#1321, Albert Gold#2696
+# Authors: Albert Gold#2696, NK#1321
 #
-# if you used it to remap your mods pls give credit for "Nhok0169" and "Albert Gold#2696"
+# if you used it to remap your mods pls give credit for "Albert Gold#2696" and "Nhok0169"
 # Special Thanks:
 #   nguen#2011 (for support)
 #   SilentNightSound#7430 (for internal knowdege so wrote the blendCorrection code)
@@ -14,6 +14,7 @@
 ##### ExtImports
 import copy
 import os
+import re
 from typing import Dict, Optional, Set, List, Tuple
 ##### EndExtImports
 
@@ -22,11 +23,14 @@ from ....constants.FileExt import FileExt
 from ....constants.IniConsts import IniKeywords
 from ....tools.TextTools import TextTools
 from ....tools.DictTools import DictTools
+from ....tools.HashTools import HashTools
+from ....tools.files.FileService import FileService
 from .GIMIFixer import GIMIFixer
 from ..iniParsers.GIMIObjParser import GIMIObjParser
 from ...iftemplate.IfContentPart import IfContentPart
 from ...iftemplate.IfTemplate import IfTemplate
 from .regEditFilters.BaseRegEditFilter import BaseRegEditFilter
+from .regEditFilters.RegEditFilter import RegEditFilter
 from .regEditFilters.RegTexAdd import RegTexAdd
 from ..texEditors.TexCreator import TexCreator
 from ...IniSectionGraph import IniSectionGraph
@@ -71,6 +75,12 @@ class GIMIObjReplaceFixer(GIMIFixer):
 
         **Default**: ``True``
 
+    postModelRegEditFilters: Optional[List[:class:`RegEditFilter`]]
+        Filters used to edit the registers of a certain :class:`IfContentPart` for the `sections`_ related to the .VB or .IB of a mod
+        Filters are executed based on the order specified in the list. :raw-html:`<br />` :raw-html:`<br />`
+
+        **Default**: ``None``
+
     Attributes
     ----------
     preRegEditOldObj: :class:`bool`
@@ -92,8 +102,8 @@ class GIMIObjReplaceFixer(GIMIFixer):
     """
 
     def __init__(self, parser: GIMIObjParser, preRegEditFilters: Optional[List[BaseRegEditFilter]] = None, postRegEditFilters: Optional[List[BaseRegEditFilter]] = None,
-                 preRegEditOldObj: bool = True):
-        super().__init__(parser)
+                 preRegEditOldObj: bool = True, postModelRegEditFilters: Optional[List[RegEditFilter]] = None):
+        super().__init__(parser, postModelRegEditFilters = postModelRegEditFilters)
         self._texInds: Dict[str, Dict[str, int]] = {}
         self._texEditRemapNames: Dict[str, Dict[str, str]] = {}
         self._texAddRemapNames: Dict[str, Dict[str, str]] = {}
@@ -115,8 +125,8 @@ class GIMIObjReplaceFixer(GIMIFixer):
         for filter in filters:
             if (isinstance(filter, RegTexAdd)):
                 self.addedTextures = DictTools.combine(self.addedTextures, copy.deepcopy(filter.textures), 
-                                                       lambda srcObjTextures, currentObjTextures: DictTools.combine(srcObjTextures, currentObjTextures, 
-                                                                                                                    lambda srcTexData, currentTexData: currentTexData))
+                                                       lambda modObj, srcObjTextures, currentObjTextures: DictTools.combine(srcObjTextures, currentObjTextures, 
+                                                                                                                    lambda reg, srcTexData, currentTexData: currentTexData))
 
     @property
     def preRegEditFilters(self):
@@ -192,7 +202,13 @@ class GIMIObjReplaceFixer(GIMIFixer):
             The new name for the `section`_
         """
 
-        name = name[:-len(objName)] + TextTools.capitalize(newObjName.lower())
+        name = TextTools.reverse(name)
+    
+        nameParts = re.split(TextTools.reverse(objName), name, flags = re.IGNORECASE, maxsplit = 1)
+        name = TextTools.reverse(TextTools.capitalizeOnlyFirstChar(newObjName)).join(nameParts)
+    
+        name = TextTools.reverse(name)
+
         return self._iniFile.getRemapFixName(name, modName = modName)
     
     def getTexResourceRemapFixName(self, texTypeName: str, oldModName: str, newModName: str, objName: str, addInd: bool = False) -> str:
@@ -252,9 +268,9 @@ class GIMIObjReplaceFixer(GIMIFixer):
     def getObjHashType(self):
         return "ib"
     
-    def editRegisters(self, modName: str, part: IfContentPart, obj: str, sectionName: str, filters: List[BaseRegEditFilter]):
+    def editTexRegisters(self, modName: str, part: IfContentPart, obj: str, sectionName: str, filters: List[BaseRegEditFilter]):
         """
-        Edits the registers for a :class:`IfContentPart`
+        Edits the registers for a :class:`IfContentPart` in the texture related `section`_
 
         .. note::
             For details on steps of how the registers are editted, see :class:`GIMIObjReplaceFixer`
@@ -262,7 +278,7 @@ class GIMIObjReplaceFixer(GIMIFixer):
         Parameters
         ----------
         modName: :class:`str`
-            The name of the mod
+            The name of the mod to fix to
 
         part: :class:`IfContentPart`
             The part that is being editted
@@ -325,10 +341,10 @@ class GIMIObjReplaceFixer(GIMIFixer):
             texEditSections.add(texEditSection)
         
     
-    def fillObjNonBlendSection(self, modName: str, sectionName: str, part: IfContentPart, partIndex: int, linePrefix: str, origSectionName: str, objName: str, newObjName: str):
+    def fillObjOtherHashIndexSection(self, modName: str, sectionName: str, part: IfContentPart, partIndex: int, linePrefix: str, origSectionName: str, objName: str, newObjName: str):
         """
-        Creates the **content part** of an :class:`IfTemplate` for the new sections created by this fix that are not related to the ``[TextureOverride.*Blend.*]`` `sections`_
-        of some mod object, where the original `section` comes from a different mod object
+        Creates the **content part** of an :class:`IfTemplate` for the new sections created by this fix where the `sections`_ reference some hash or index and the `section`_ is not
+        explictely captured by the fixer. The original `sections`_ may come from a different mod object.
 
         .. tip::
             For more info about an 'IfTemplate', see :class:`IfTemplate`
@@ -369,7 +385,7 @@ class GIMIObjReplaceFixer(GIMIFixer):
         preRegEditObj = objName if (self.preRegEditOldObj) else newObjName
 
         newPart = copy.deepcopy(part)
-        self.editRegisters(modName, newPart, preRegEditObj, sectionName, self._preRegEditFilters)
+        self.editTexRegisters(modName, newPart, preRegEditObj, sectionName, self._preRegEditFilters)
 
         for varName, varValue, keyInd, orderInd in newPart:
             # filling in the hash
@@ -388,7 +404,7 @@ class GIMIObjReplaceFixer(GIMIFixer):
                 newIndex = self._getIndex(newObjName.lower(), modName)
                 newPart.src[varName][keyInd] = (orderInd, f"{newIndex}")
 
-        self.editRegisters(modName, newPart, newObjName, sectionName, self._postRegEditFilters)
+        self.editTexRegisters(modName, newPart, newObjName, sectionName, self._postRegEditFilters)
         
         addFix = newPart.toStr(linePrefix = linePrefix)
         if (addFix != ""):
@@ -464,7 +480,7 @@ class GIMIObjReplaceFixer(GIMIFixer):
 
         return addFix
     
-    def _getTexEditFile(self, file: str, texInd: int, modName: str = "") -> str:
+    def getTexEditFile(self, file: str, texName: str, modObj: str, modName: str = "") -> str:
         """
         Makes the file path for an editted texture
 
@@ -473,8 +489,11 @@ class GIMIObjReplaceFixer(GIMIFixer):
         texFile: :class:`str`
             The file path to the original .dds file
 
-        texInd: :class:`int`
-            The index for the type of texture being editted
+        texName: :class:`str`
+            The name for the type of texture
+
+        modObj: :class:`str`
+            The name of the mod object the texture file belongs to
 
         modName: :class:`str`
             The name of the mod to fix to
@@ -485,35 +504,33 @@ class GIMIObjReplaceFixer(GIMIFixer):
             The file path of the fixed RemapTex.dds file
         """
 
-        texFolder = os.path.dirname(file)
-        texName = os.path.basename(file)
-        texName = texName.rsplit(".", 1)[0]
+        basename = os.path.basename(file)
+        ind = f"{HashTools.base64DeterministicShortUniqueHash(basename)} {HashTools.base64DeterministicShortUniqueHash(texName)}"
 
-        return os.path.join(texFolder, f"{self._iniFile.getRemapTexName(texName, modName = modName)}{texInd}{FileExt.DDS.value}")
+        texFolder = os.path.dirname(file)
+        modName = f"{modName}{TextTools.capitalize(modObj)}"
+        return os.path.join(texFolder, f"{self._iniFile.getRemapTexName('', modName = modName)}{ind}{FileExt.DDS.value}")
     
     # _fixEdittedTextures(modName, fix): get the fix string for editted textures
     def _fixEdittedTextures(self, modName: str, fix: str = ""):
         self._iniFile.texEditModels.clear()
 
         # rebuild all the models and the section graphs
-        texInd = 0
         for texName in self._referencedTexEditSections:
             referencedSections = list(self._referencedTexEditSections[texName])
             referencedSections.sort()
 
             texGraph = self._parser.getTexGraph(texName)
             if (texGraph is None):
-                texInd += 1
                 continue
 
-            texGraph.build(newTargetSections = referencedSections)
+            texGraph.build(newTargetSections = referencedSections, newAllSections = self._iniFile.sectionIfTemplates)
             texEditor = self._parser.getTexEditor(texName)
             if (texEditor is None):
-                texInd += 1
                 continue
-
-            self._parser._makeTexModels(texName, texGraph, texEditor, getFixedFile = lambda file, modName: self._getTexEditFile(file, texInd, modName = modName))
-            texInd += 1
+            
+            modObjName = self._parser.texEditRegs[texName][0]
+            self._parser._makeTexModels(texName, texGraph, texEditor, getFixedFile = lambda file, modName: self.getTexEditFile(file, texName, modObjName, modName = modName))
 
         texEditInd = 0
         referencedTexEditLen = len(self._referencedTexEditSections)
@@ -550,6 +567,9 @@ class GIMIObjReplaceFixer(GIMIFixer):
 
             texEditInd += 1
 
+        if (fix and fix[-1] == "\n"):
+            fix = fix[:-1]
+
         return fix
     
     # _makeTexAddResourceIfTemplate(texName, modName, oldModName, modObj): Creates the IfTemplate for an added texture
@@ -566,11 +586,8 @@ class GIMIObjReplaceFixer(GIMIFixer):
             sectionName = self.getTexResourceRemapFixName(texName, oldModName, modName, modObj)
             self._texAddRemapNames[texName][modObj] = sectionName
 
-        filePartName = sectionName
-        if (sectionName.startswith(IniKeywords.Resource.value)):
-            filePartName = filePartName[len(IniKeywords.Resource.value):]
-
-        filename = f"{self._iniFile.getRemapTexName(filePartName, modName = modName)}{FileExt.DDS.value}"
+        filePartName = f"{modName}{TextTools.capitalize(modObj)}{TextTools.capitalize(texName)}"
+        filename = f"{self._iniFile.getRemapTexName(filePartName)}{FileExt.DDS.value}"
 
         return IfTemplate([
             IfContentPart({"filename": [(0, filename)]}, 0)
@@ -616,6 +633,31 @@ class GIMIObjReplaceFixer(GIMIFixer):
 
         return fix
     
+    # _fixDownloadResources(fix): get the fix string for downloaded files
+    def _fixDownloadedResources(self, fix: str = "", includeEndNewLine = False):
+        fix = super()._fixDownloadedResources(fix = fix, includeEndNewLine = True)
+
+        downloadAdded = False
+        referencedDownloads = self._parser._objReferencedDownloads
+
+        for section in referencedDownloads:
+            registers = referencedDownloads[section]
+
+            for reg in registers:
+                modObj, sectionName = registers[reg]
+
+                ifTemplate = self._iniFile.sectionIfTemplates.get(sectionName)
+                fix += self.fillIfTemplate("", sectionName, ifTemplate, lambda modName, sectionName, part, partIndex, linePrefix, origSectionName: f"{part.toStr(linePrefix = linePrefix)}\n")
+                fix += "\n"
+
+                if (not downloadAdded):
+                    downloadAdded = True
+
+        if (not includeEndNewLine and downloadAdded and fix and fix[-1] == "\n"):
+            fix = fix[:-1]
+
+        return fix
+
     def fixMod(self, modName: str, fix: str = "") -> str:
         self._texEditRemapNames = {}
         self._referencedTexEditSections = {}
@@ -637,5 +679,6 @@ class GIMIObjReplaceFixer(GIMIFixer):
 
         if (fix and fix[-1] != "\n"):
             fix += "\n"
+
         return fix
 ##### EndScript
\ No newline at end of file
